@@ -57,10 +57,12 @@ G_DEFINE_TYPE (IMSettingsClient, imsettings_client, G_TYPE_OBJECT);
 struct _IMSettingsClientPrivate {
 	GDBusProxy *proxy;
 	gchar      *locale;
+	gchar      *desktop;
 };
 enum {
 	PROP_0,
 	PROP_LOCALE,
+	PROP_DESKTOP,
 	LAST_PROP
 };
 
@@ -108,6 +110,10 @@ imsettings_client_set_property(GObject      *object,
 		    imsettings_client_set_locale(IMSETTINGS_CLIENT (object),
 						 g_value_get_string(value));
 		    break;
+	    case PROP_DESKTOP:
+		    imsettings_client_set_desktop(IMSETTINGS_CLIENT (object),
+						  g_value_get_string(value));
+		    break;
 	    default:
 		    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		    break;
@@ -125,6 +131,10 @@ imsettings_client_get_property(GObject    *object,
 		    g_value_set_string(value,
 				       imsettings_client_get_locale(IMSETTINGS_CLIENT (object)));
 		    break;
+	    case PROP_DESKTOP:
+		    g_value_set_string(value,
+				       imsettings_client_get_desktop(IMSETTINGS_CLIENT (object)));
+		    break;
 	    default:
 		    G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		    break;
@@ -140,6 +150,7 @@ imsettings_client_finalize(GObject *object)
 	if (priv->proxy)
 		g_object_unref(priv->proxy);
 	g_free(priv->locale);
+	g_free(priv->desktop);
 
 	if (G_OBJECT_CLASS (imsettings_client_parent_class)->finalize)
 		G_OBJECT_CLASS (imsettings_client_parent_class)->finalize(object);
@@ -163,6 +174,12 @@ imsettings_client_class_init(IMSettingsClientClass *klass)
 							    _("Locale to get the imsettings information"),
 							    NULL,
 							    G_PARAM_READWRITE));
+	g_object_class_install_property(object_class, PROP_DESKTOP,
+					g_param_spec_string("desktop",
+							    _("Desktop"),
+							    _("Current desktop"),
+							    NULL,
+							    G_PARAM_READWRITE));
 }
 
 static void
@@ -174,6 +191,7 @@ imsettings_client_init(IMSettingsClient *client)
 
 	priv->proxy = NULL;
 	priv->locale = NULL;
+	priv->desktop = NULL;
 }
 
 G_INLINE_FUNC gboolean
@@ -323,6 +341,54 @@ imsettings_client_get_locale(IMSettingsClient *client)
 	return priv->locale;
 }
 
+/**
+ * imsettings_client_set_desktop:
+ * @client: a #IMSettingsClient.
+ * @desktop: a current desktop name going to operate.
+ *
+ * Update the desktop name in @client to @desktop. if @desktop is %NULL,
+ * the desktop detection will be done in the server side.
+ *
+ * Returns: %TRUE if successfully updated, otherwise %FALSE.
+ */
+gboolean
+imsettings_client_set_desktop(IMSettingsClient *client,
+			      const gchar      *desktop)
+{
+	IMSettingsClientPrivate *priv;
+
+	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), FALSE);
+
+	priv = client->priv;
+	g_free(priv->desktop);
+	priv->desktop = g_strdup(desktop);
+
+	g_object_notify(G_OBJECT (client), "desktop");
+
+	return TRUE;
+}
+
+/**
+ * imsettings_client_get_desktop:
+ * @client: a #IMSettingsClient.
+ *
+ * Obtains current desktop information in @client.
+ *
+ * Returns: a reference to the desktop name in @client. it shouldn't be freed
+ *          in applications.
+ */
+const gchar *
+imsettings_client_get_desktop(IMSettingsClient *client)
+{
+	IMSettingsClientPrivate *priv;
+
+	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), NULL);
+
+	priv = client->priv;
+
+	return priv->desktop;
+}
+
 /**
  * imsettings_client_get_version:
  * @client: a #IMSettingsClient.
@@ -885,6 +951,7 @@ imsettings_client_switch_im(IMSettingsClient  *client,
 	gboolean retval = FALSE;
 	GVariant *value;
 	gchar *m;
+	const gchar *desktop;
 
 	g_return_val_if_fail (IMSETTINGS_IS_CLIENT (client), FALSE);
 
@@ -892,17 +959,31 @@ imsettings_client_switch_im(IMSettingsClient  *client,
 		m = g_strdup(IMSETTINGS_NONE_CONF);
 	else
 		m = g_strdup(module);
+	desktop = imsettings_client_get_desktop(client);
 
 	proxy = imsettings_client_get_proxy(client);
-	value = g_dbus_proxy_call_sync(proxy,
-				       "SwitchIM",
-				       g_variant_new("(ssb)",
-						     imsettings_client_get_locale(client),
-						     m, update_xinputrc),
-				       G_DBUS_CALL_FLAGS_NONE,
-				       -1,
-				       cancellable,
-				       error);
+	if (desktop) {
+		value = g_dbus_proxy_call_sync(proxy,
+					       "SwitchIMNoDesktopDetection",
+					       g_variant_new("(sssb)",
+							     imsettings_client_get_locale(client),
+							     desktop,
+							     m, update_xinputrc),
+					       G_DBUS_CALL_FLAGS_NONE,
+					       -1,
+					       cancellable,
+					       error);
+	} else {
+		value = g_dbus_proxy_call_sync(proxy,
+					       "SwitchIM",
+					       g_variant_new("(ssb)",
+							     imsettings_client_get_locale(client),
+							     m, update_xinputrc),
+					       G_DBUS_CALL_FLAGS_NONE,
+					       -1,
+					       cancellable,
+					       error);
+	}
 	if (value != NULL) {
 		g_variant_get(value, "(b)", &retval);
 		g_variant_unref(value);
@@ -935,6 +1016,7 @@ imsettings_client_switch_im_start(IMSettingsClient    *client,
 {
 	GDBusProxy *proxy;
 	gchar *m;
+	const gchar *desktop;
 
 	g_return_if_fail (IMSETTINGS_IS_CLIENT (client));
 
@@ -942,18 +1024,33 @@ imsettings_client_switch_im_start(IMSettingsClient    *client,
 		m = g_strdup(IMSETTINGS_NONE_CONF);
 	else
 		m = g_strdup(module);
+	desktop = imsettings_client_get_desktop(client);
 
 	proxy = imsettings_client_get_proxy(client);
-	g_dbus_proxy_call(proxy,
-			  "SwitchIM",
-			  g_variant_new("(ssb)",
-					imsettings_client_get_locale(client),
-					m, update_xinputrc),
-			  G_DBUS_CALL_FLAGS_NONE,
-			  -1,
-			  cancellable,
-			  callback,
-			  user_data);
+	if (desktop) {
+		g_dbus_proxy_call(proxy,
+				  "SwitchIMNoDesktopDetection",
+				  g_variant_new("(sssb)",
+						imsettings_client_get_locale(client),
+						desktop,
+						m, update_xinputrc),
+				  G_DBUS_CALL_FLAGS_NONE,
+				  -1,
+				  cancellable,
+				  callback,
+				  user_data);
+	} else {
+		g_dbus_proxy_call(proxy,
+				  "SwitchIM",
+				  g_variant_new("(ssb)",
+						imsettings_client_get_locale(client),
+						m, update_xinputrc),
+				  G_DBUS_CALL_FLAGS_NONE,
+				  -1,
+				  cancellable,
+				  callback,
+				  user_data);
+	}
 	g_free(m);
 }
 
